// Copyright © 2020 Amazon Web Services
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//      http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
package org.opengroup.osdu.register.provider.aws.subscriber;

import com.amazonaws.services.dynamodbv2.datamodeling.DynamoDBDeleteExpression;
import com.amazonaws.services.dynamodbv2.model.ConditionalCheckFailedException;
import com.amazonaws.services.sns.AmazonSNS;
import org.junit.Before;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.Mockito;
import org.mockito.runners.MockitoJUnitRunner;
import org.opengroup.osdu.core.aws.dynamodb.DynamoDBQueryHelper;
import org.opengroup.osdu.core.common.logging.JaxRsDpsLog;
import org.opengroup.osdu.core.common.model.http.AppException;
import org.opengroup.osdu.core.common.model.http.DpsHeaders;
import org.opengroup.osdu.register.ddms.model.Ddms;
import org.opengroup.osdu.register.provider.aws.config.AwsServiceConfig;
import org.opengroup.osdu.register.provider.aws.ddms.DdmsDoc;
import org.opengroup.osdu.register.provider.aws.util.DocUtil;
import org.opengroup.osdu.register.provider.aws.util.TestUtils;
import org.opengroup.osdu.register.subscriber.model.*;
import org.opengroup.osdu.register.utils.Constants;

import java.nio.ByteBuffer;
import java.sql.Timestamp;
import java.util.ArrayList;
import java.util.List;

import static org.junit.Assert.assertEquals;
import static org.mockito.Mockito.lenient;
import static org.mockito.Mockito.when;

@RunWith(MockitoJUnitRunner.class)
public class AwsSubscriptionRepoTest {

    @Mock
    private DpsHeaders dpsHeaders;

    @Mock
    DynamoDBQueryHelper dynamoDBQueryHelper;

    @Mock
    private JaxRsDpsLog logger;

    @InjectMocks
    private AwsSubscriptionRepo repo;


    @Mock
    private DocUtil docUtil;


    @Mock
    private KmsHelper kmsHelper;

    @Mock
    private AwsServiceConfig serviceConfig;

    @Mock
    private SubscriptionDoc subscriptionDoc;

    @Mock
    private SubscriptionHelper snsHelper;


    @Before
    public void init() {
        lenient().doReturn(TestUtils.getDataPartitionId()).when(dpsHeaders).getPartitionId();



    }

    @Test
    public void getSubscription_success(){
        Subscription sub = createMockSubscription();
        SubscriptionDoc subDoc = createMockSubscriptionDoc();
        String id = TestUtils.getSub_id();

        Mockito.when(dynamoDBQueryHelper.loadByPrimaryKey(SubscriptionDoc.class, id)).thenReturn(subDoc);
        Mockito.when(kmsHelper.decrypt(Mockito.any(ByteBuffer.class))).thenReturn(TestUtils.getSub_secretValue_string());
        Mockito.when((docUtil.createSecret(Mockito.anyString(),Mockito.anyString()))).thenReturn(createTestSecret());


        Subscription resultSub = repo.get(id);
        sub.setCreatedOnEpoch(resultSub.getCreatedOnEpoch());
        assertEquals(sub, resultSub);

    }

    @Test
    public void getSubscription_throw404_whenIdoesNotExist(){
        String id = TestUtils.getSub_id();
        AppException e = new AppException(404,"","");
        Mockito.doThrow(e).when(dynamoDBQueryHelper).loadByPrimaryKey(SubscriptionDoc.class, id);
        Boolean result=true;
        try {
            Subscription  resultSub = repo.get(id);
        } catch (AppException ex) {
            assertEquals(404, ex.getError().getCode());

        }
    }

    @Test
    public void createSubscription_success() throws Exception {
        Subscription s = createMockSubscription();
        //Mockito.when(snsHelper.createPushSubscription(Mockito.anyString(),Mockito.anyString())).thenReturn(TestUtils.getSub_sns_subscriptionArn());
        Mockito.when(kmsHelper.encrypt(Mockito.anyString())).thenReturn(TestUtils.getSub_encrypted_secret_value_bytebuffer());
        Mockito.doNothing().when(dynamoDBQueryHelper).saveWithHashCondition(Mockito.any(SubscriptionDoc.class),Mockito.anyString());



        Subscription resultSub = repo.create(s);
        s.setCreatedOnEpoch(resultSub.getCreatedOnEpoch());
        assertEquals(s, resultSub);
    }

    @Test
    public void createSubscription_throw409_ifIdalreadyexists() throws Exception {
        Subscription s = createMockSubscription();
        AppException e =new AppException(409, "Conflict", String.format("A Subscription already exists with the same id: %s", s.getId()));
        Mockito.doThrow(e).when(dynamoDBQueryHelper).saveWithHashCondition(Mockito.any(SubscriptionDoc.class),Mockito.anyString());
        try {
            Subscription resultSub = repo.create(s);
        }
        catch (AppException ex) {
            assertEquals(409, ex.getError().getCode());
        }


    }

    @Test
    public void delete_success() {

        SubscriptionDoc subDoc = createMockSubscriptionDoc();
        String id = TestUtils.getSub_id();
        Mockito.when(dynamoDBQueryHelper.loadByPrimaryKey(SubscriptionDoc.class, id)).thenReturn(subDoc);
        Mockito.doNothing().when(dynamoDBQueryHelper).deleteByObjectWithCondition(Mockito.any(SubscriptionDoc.class),Mockito.any(DynamoDBDeleteExpression.class));
        Mockito.doNothing().when(snsHelper).deletePushSubscription(Mockito.anyString());

        boolean result = repo.delete(id);
        assertEquals(true,result);
    }

    @Test
    public void delete_throws404_whenIddoesNotExist() {
        SubscriptionDoc subDoc = createMockSubscriptionDoc();
        String id = TestUtils.getSub_id();
        Mockito.when(dynamoDBQueryHelper.loadByPrimaryKey(SubscriptionDoc.class, id)).thenReturn(subDoc);
        ConditionalCheckFailedException e = new ConditionalCheckFailedException("Failed to delete");
        Mockito.doThrow(e).when(dynamoDBQueryHelper).deleteByObjectWithCondition(Mockito.any(SubscriptionDoc.class),Mockito.any(DynamoDBDeleteExpression.class));
        try {
           repo.delete(id);
        } catch (AppException ex) {
            assertEquals(404, ex.getError().getCode());
        }
    }


    @Test
    public void patch_success() {

        Subscription s = createMockSubscription();
        SubscriptionDoc subDoc = createMockSubscriptionDoc();
        String id = TestUtils.getSub_id();

        Secret secret = Mockito.mock(Secret.class);
        Mockito.when(dynamoDBQueryHelper.loadByPrimaryKey(SubscriptionDoc.class, id)).thenReturn(subDoc);
        Mockito.when(secret.getSecretType()).thenReturn(TestUtils.getSub_secretType());
        Mockito.when(kmsHelper.encrypt(Mockito.anyString())).thenReturn(TestUtils.getSub_updated_encrypted_secretValue_bytebuffer());
        Mockito.doNothing().when(dynamoDBQueryHelper).save(Mockito.any(SubscriptionDoc.class));

        boolean result = repo.patch(s,secret);
        assertEquals(true,result);

    }

    @Test
    public void patch_throwException_ifcannotBeUpdated() {

        Subscription s = createMockSubscription();
        SubscriptionDoc subDoc = createMockSubscriptionDoc();
        String id = TestUtils.getSub_id();
        AppException e = new AppException(500,"","Error while saving the updated secret");

        Secret secret = Mockito.mock(Secret.class);
        Mockito.when(dynamoDBQueryHelper.loadByPrimaryKey(SubscriptionDoc.class, id)).thenReturn(subDoc);
        Mockito.when(secret.getSecretType()).thenReturn(TestUtils.getSub_secretType());
        Mockito.when(kmsHelper.encrypt(Mockito.anyString())).thenReturn(TestUtils.getSub_updated_encrypted_secretValue_bytebuffer());
        Mockito.doThrow(e).when(dynamoDBQueryHelper).save(Mockito.any(SubscriptionDoc.class));

        try {
            boolean result = repo.patch(s,secret);
        } catch (AppException ex) {
            assertEquals(500, ex.getError().getCode());
            assertEquals("Error while saving the updated secret", ex.getError().getMessage());
        }


    }

    @Test
    public void patch_throw404_ifIdoesNotExist() {
        Subscription s = createMockSubscription();
        String id = TestUtils.getSub_id();

        Secret secret = Mockito.mock(Secret.class);

        AppException e = new AppException(404,"","");
        Mockito.doThrow(e).when(dynamoDBQueryHelper).loadByPrimaryKey(SubscriptionDoc.class, id);
        Boolean result=true;
        try {
            result = repo.patch(s,secret);
        } catch (AppException ex) {
            assertEquals(404, ex.getError().getCode());
        }

    }
    @Test
    public void getAll_success() throws Exception {

        Subscription s1 = createMockSubscription();
        Subscription s2 = createMockSubscription();
        s2.setId("SubTestId2");
        List<Subscription> subList = new ArrayList<Subscription>();
        SubscriptionDoc doc1 = SubscriptionDoc.mapFrom(s1,TestUtils.getDataPartitionId(),TestUtils.getSub_sns_subscriptionArn(), TestUtils.getSub_encrypted_secret_value_bytebuffer());
        SubscriptionDoc doc2 = SubscriptionDoc.mapFrom(s2,TestUtils.getDataPartitionId(),TestUtils.getSub_sns_subscriptionArn(), TestUtils.getSub_encrypted_secret_value_bytebuffer());


        ArrayList<SubscriptionDoc> subDocs = new ArrayList<SubscriptionDoc>();
        subDocs.add(doc1);
        subDocs.add(doc2);

        Mockito.when(dynamoDBQueryHelper.scanTable(Mockito.any(Class.class),Mockito.anyString(),Mockito.anyMap())).thenReturn(subDocs);
        Mockito.when(kmsHelper.decrypt(Mockito.any(ByteBuffer.class))).thenReturn(TestUtils.getSub_secretValue_string());
        Mockito.when((docUtil.createSecret(Mockito.anyString(),Mockito.anyString()))).thenReturn(createTestSecret());

        List<Subscription> resultSubs = repo.getAll();
        s1.setCreatedOnEpoch(resultSubs.get(0).getCreatedOnEpoch());
        s2.setCreatedOnEpoch(resultSubs.get(1).getCreatedOnEpoch());

        subList.add(s1);
        subList.add(s2);
        assertEquals(subList, resultSubs);


    }

    @Test
    public void getAll_fail() throws Exception {

        Subscription s1 = createMockSubscription();
        Subscription s2 = createMockSubscription();
        s2.setId("SubTestId2");
        List<Subscription> subList = new ArrayList<Subscription>();
        SubscriptionDoc doc1 = SubscriptionDoc.mapFrom(s1,TestUtils.getDataPartitionId(),TestUtils.getSub_sns_subscriptionArn(), TestUtils.getSub_encrypted_secret_value_bytebuffer());
        SubscriptionDoc doc2 = SubscriptionDoc.mapFrom(s2,TestUtils.getDataPartitionId(),TestUtils.getSub_sns_subscriptionArn(), TestUtils.getSub_encrypted_secret_value_bytebuffer());


        ArrayList<SubscriptionDoc> subDocs = new ArrayList<SubscriptionDoc>();
        subDocs.add(doc1);
        subDocs.add(doc2);
        AppException e = new AppException(500,"","Error while getting ALL subscriptions");
        Mockito.doThrow(e).when(dynamoDBQueryHelper).scanTable(Mockito.any(Class.class),Mockito.anyString(),Mockito.anyMap());
        try {
            List<Subscription> resultSubs = repo.getAll();
        } catch (AppException ex) {
            assertEquals(500, ex.getError().getCode());
            assertEquals("Error while getting ALL subscriptions", ex.getError().getMessage());
        }



    }

    @Test
    public void query_success() throws Exception {
        Subscription s1 =  createMockSubscription();
        Subscription s2 =  createMockSubscription();
        s2.setId("SubTestId2");
        List<Subscription> subList = new ArrayList<Subscription>();
        subList.add(s1);
        subList.add(s2);

        SubscriptionDoc doc1 = SubscriptionDoc.mapFrom(s1,TestUtils.getDataPartitionId(),TestUtils.getSub_sns_subscriptionArn(),TestUtils.getSub_encrypted_secret_value_bytebuffer());
        SubscriptionDoc doc2 = SubscriptionDoc.mapFrom(s2,TestUtils.getDataPartitionId(),TestUtils.getSub_sns_subscriptionArn(),TestUtils.getSub_encrypted_secret_value_bytebuffer());


        List<SubscriptionDoc> subDocsList = new ArrayList<SubscriptionDoc>();
        subDocsList.add(doc1);
        subDocsList.add(doc2);
        //Mockito.when(kmsHelper.decrypt(Mockito.anyString())).thenReturn(TestUtils.getSub_secretValue());
        Mockito.when(docUtil.getSubscriptionList(Mockito.any(DynamoDBQueryHelper.class),Mockito.any(SubscriptionDoc.class))).thenReturn(subList);

        List<Subscription> resultSubList = repo.query(TestUtils.getSub_notificationId());


        subList.get(0).setCreatedOnEpoch(resultSubList.get(0).getCreatedOnEpoch());
        subList.get(1).setCreatedOnEpoch(resultSubList.get(1).getCreatedOnEpoch());

        assertEquals(subList, resultSubList);

    }

        private Subscription createMockSubscription() {
        Subscription sub = new Subscription();
        sub.setId(TestUtils.getSub_id());
        sub.setSecret(new HmacSecret(TestUtils.getSub_secretValue_string()));
        sub.setNotificationId(TestUtils.getSub_id());
        sub.setTopic(TestUtils.getSub_topicName());




        return sub;
    }



    private SubscriptionDoc createMockSubscriptionDoc() {
        SubscriptionDoc subDoc = new SubscriptionDoc();
        subDoc.setId(TestUtils.getSub_id());
        subDoc.setSecretType(TestUtils.getSub_secretType());
        subDoc.setNotificationId(TestUtils.getSub_id());
        subDoc.setTopic(TestUtils.getSub_topicName());
        subDoc.setSecretValue(TestUtils.getSub_secretValue_bytebuffer());
        subDoc.setCreatedOnEpoch((new Timestamp(System.currentTimeMillis())).toString());
        subDoc.setSnssubscriptionArn(TestUtils.getSub_sns_subscriptionArn());

        return subDoc;
    }

    public Secret createTestSecret()
    {
        Secret secret;
        HmacSecret hmacSecret = new HmacSecret();
        hmacSecret.setValue(TestUtils.getSub_secretValue_string());
        secret = hmacSecret;
        return secret;
    }

}
