/*
// Copyright © Microsoft Corporation
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//      http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

package org.opengroup.osdu.register.provider.azure.util;

import com.azure.security.keyvault.keys.cryptography.CryptographyClient;
import com.azure.security.keyvault.keys.cryptography.models.DecryptResult;
import com.azure.security.keyvault.keys.cryptography.models.EncryptResult;
import com.azure.security.keyvault.keys.cryptography.models.EncryptionAlgorithm;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.api.extension.ExtendWith;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.junit.jupiter.MockitoExtension;
import org.opengroup.osdu.register.provider.azure.di.AzureBootstrapConfig;

import static org.junit.jupiter.api.Assertions.assertEquals;
import static org.mockito.Mockito.doReturn;
import static org.mockito.Mockito.times;
import static org.mockito.Mockito.verify;
import static org.mockito.Mockito.when;

@ExtendWith(MockitoExtension.class)
public class CryptographyUtilTest {

    private static final String plainText = "plain-text";
    private static final byte[] bytesArray = new byte[] {1,2,3,4};
    private static final String encodedData = "AQIDBA==";
    private static final String keyIdentifier = "https://haaggarw-test-vault.vault.azure.net/keys/test-key/key-version";

    @InjectMocks
    private CryptographyUtil cryptographyUtil;

    @Mock
    private CryptographyClient cryptographyClient;

    @Mock
    private EncryptResult encryptResult;

    @Mock
    private DecryptResult decryptResult;

    @Mock
    private AzureBootstrapConfig azureBootstrapConfig;

    @Test
    public void shouldSuccessfullyReturnEncryptedData() {
        when(encryptResult.getCipherText()).thenReturn(bytesArray);
        doReturn(encryptResult).when(cryptographyClient).encrypt(EncryptionAlgorithm.RSA_OAEP, plainText.getBytes());

        String result = cryptographyUtil.encryptData(plainText);

        assertEquals(encodedData, result);
        verify(cryptographyClient, times(1)).encrypt(EncryptionAlgorithm.RSA_OAEP, plainText.getBytes());
        verify(encryptResult, times(1)).getCipherText();
    }

    @Test
    public void shouldSuccessfullyReturnDecryptedData() {
        when(decryptResult.getPlainText()).thenReturn(plainText.getBytes());
        doReturn(decryptResult).when(cryptographyClient).decrypt(EncryptionAlgorithm.RSA_OAEP, bytesArray);

        String result = cryptographyUtil.decryptData(encodedData);

        assertEquals(plainText, result);
        verify(cryptographyClient, times(1)).decrypt(EncryptionAlgorithm.RSA_OAEP, bytesArray);
        verify(decryptResult, times(1)).getPlainText();
    }

    @Test
    public void shouldReturnKeyName() {
        when(azureBootstrapConfig.getKeyIdentifier()).thenReturn(keyIdentifier);
        String result = cryptographyUtil.getKeyName();

        assertEquals("test-key", result);
        verify(azureBootstrapConfig, times(1)).getKeyIdentifier();
    }

    @Test
    public void shouldReturnKeyVersion() {
        when(azureBootstrapConfig.getKeyIdentifier()).thenReturn(keyIdentifier);

        String result = cryptographyUtil.getKeyVersion();
        assertEquals("key-version", result);
        verify(azureBootstrapConfig, times(1)).getKeyIdentifier();
    }

}
*/
