/*
 * Copyright 2017-2020, Schlumberger
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.opengroup.osdu.register.api;

import javax.validation.constraints.NotBlank;
import org.opengroup.osdu.register.ddms.model.Ddms;
import org.opengroup.osdu.register.provider.interfaces.ddms.IDdmsRepository;
import org.opengroup.osdu.register.ddms.services.ConsumptionService;
import org.opengroup.osdu.core.common.model.http.AppException;
import org.opengroup.osdu.register.logging.AuditLog;
import org.opengroup.osdu.register.utils.ServiceRole;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.context.annotation.RequestScope;

import javax.validation.Valid;
import javax.validation.constraints.Pattern;
import java.net.URI;
import java.util.List;

@RestController
@RequestMapping("/")
@RequestScope
@Validated
public class DdmsApi {

    @Autowired
    private IDdmsRepository dmsRepository;
    @Autowired
    private AuditLog auditLog;
    @Autowired
    private ConsumptionService consumptionService;

    @PostMapping("ddms")
    @PreAuthorize("@authorizationFilter.hasAnyPermission('" + ServiceRole.EDITOR + "', '" + ServiceRole.ADMIN + "', '" + ServiceRole.OPS + "')")
    public ResponseEntity<Ddms> postDMS(@RequestBody @Valid Ddms spec) {
        dmsRepository.create(spec);
        auditLog.createDmsSuccess(spec.getId());
        return new ResponseEntity<>(spec, HttpStatus.CREATED);
    }

    @GetMapping("ddms/{id}")
    @PreAuthorize("@authorizationFilter.hasAnyPermission('" + ServiceRole.OPS + "', '" + ServiceRole.ADMIN + "', '" + ServiceRole.EDITOR + "', '" + ServiceRole.VIEWER + "')")
    public ResponseEntity<Ddms> getDMS(@PathVariable("id") @NotBlank @Pattern(regexp = "^[A-Za-z0-9-]{2,50}") String id) {
        Ddms result = dmsRepository.get(id);
        return new ResponseEntity<>(result, HttpStatus.OK);
    }

    @GetMapping("ddms")
    @PreAuthorize("@authorizationFilter.hasAnyPermission('" + ServiceRole.OPS + "', '" + ServiceRole.ADMIN + "', '" + ServiceRole.EDITOR + "', '" + ServiceRole.VIEWER + "')")
    public ResponseEntity<List<Ddms>> queryDMS(@RequestParam("type") @NotBlank @Pattern(regexp = "^[A-Za-z0-9]{1,50}") String type) {
        List<Ddms> result = dmsRepository.query(type);
        return new ResponseEntity<>(result, HttpStatus.OK);
    }

    @DeleteMapping("ddms/{id}")
    @PreAuthorize("@authorizationFilter.hasAnyPermission('" + ServiceRole.ADMIN + "', '" + ServiceRole.OPS + "')")
    public ResponseEntity deleteDMS(@PathVariable("id") @NotBlank @Pattern(regexp = "^[A-Za-z0-9-]{2,50}") String id) {
        boolean result = dmsRepository.delete(id);
        if (result) {
            auditLog.deleteDmsSuccess(id);
            return new ResponseEntity(HttpStatus.NO_CONTENT);
        } else {
            throw new AppException(org.apache.http.HttpStatus.SC_NOT_FOUND, "Not found", String.format("Could not delete DDMS with id %s because it could not be found", id));
        }
    }

    @GetMapping("ddms/{id}/{type}/{localid}")
    @PreAuthorize("@authorizationFilter.hasAnyPermission('" + ServiceRole.OPS + "', '" + ServiceRole.ADMIN + "', '" + ServiceRole.EDITOR + "', '" + ServiceRole.VIEWER + "')")
    public ResponseEntity redirectToDms(
            @PathVariable("id") @NotBlank @Pattern(regexp = "^[A-Za-z0-9-]{2,50}") String id,
            @PathVariable("type") @NotBlank @Pattern(regexp = "^[A-Za-z]{2,50}") String type,
            @PathVariable("localid") @NotBlank @Pattern(regexp = "^[A-Za-z0-9-]{2,50}") String localid) throws Exception {
        URI targetURIForRedirection = this.consumptionService.getSingleEntityRetrivalUri(id, type, localid);
        return ResponseEntity.status(HttpStatus.TEMPORARY_REDIRECT).location(targetURIForRedirection).build();
    }
}
