/*
 * Copyright 2017-2020, Schlumberger
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.opengroup.osdu.register.utils;

import java.util.regex.Matcher;
import java.util.regex.Pattern;

public class TimeLimitedMatcher {

    private final long REGEX_TIMEOUT = 2000L;

    public Matcher matcher(Pattern pattern, CharSequence charSequence, long timeOutInMilliseconds) {
        if (!(charSequence instanceof TimeLimitedCharSequence)) {
            charSequence = new TimeLimitedCharSequence(charSequence, timeOutInMilliseconds, pattern, charSequence);
        }
        return pattern.matcher(charSequence);
    }

    public Matcher matcher(Pattern pattern, CharSequence charSequence) {
        return matcher(pattern, charSequence, REGEX_TIMEOUT);
    }

    public class RegexTimeOutException extends RuntimeException {
        public RegexTimeOutException(String message) {
            super(message);
        }
    }

    private class TimeLimitedCharSequence implements CharSequence {
        private final CharSequence inner;
        private final long timeOutInMilliseconds;
        private final long timeOutAfterTimeStamp;
        private final Pattern pattern;
        private final CharSequence originalCharSequence;

        public TimeLimitedCharSequence(CharSequence inner, long timeOutInMilliseconds, Pattern pattern, CharSequence originalCharSequence) {
            super();
            this.inner = inner;
            this.timeOutInMilliseconds = timeOutInMilliseconds;
            this.timeOutAfterTimeStamp = System.currentTimeMillis() + timeOutInMilliseconds;
            this.pattern = pattern;
            this.originalCharSequence = originalCharSequence;
        }

        @Override
        public int length() {
            return inner.length();
        }

        @Override
        public char charAt(int index) {
            if (System.currentTimeMillis() > timeOutAfterTimeStamp) {
                StringBuilder msg = new StringBuilder();
                String original = originalCharSequence.toString();
                if (original.length() > 100) {
                    original = original.substring(0, 100) + "...";
                }
                msg.append("Regular expression timeout after ")
                        .append(timeOutInMilliseconds)
                        .append("ms for [ ")
                        .append(pattern.pattern())
                        .append(" ] operating on [ ")
                        .append(original)
                        .append(" ]");

                throw new RegexTimeOutException(msg.toString());
            }
            return inner.charAt(index);
        }

        @Override
        public CharSequence subSequence(int start, int end) {
            return new TimeLimitedCharSequence(inner.subSequence(start, end), timeOutAfterTimeStamp - System.currentTimeMillis(), pattern, originalCharSequence);
        }

        @Override
        public String toString() {
            return inner.toString();
        }
    }
}
