/*
 * Copyright 2017-2020, Schlumberger
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.opengroup.osdu.register.api;

import org.apache.http.HttpStatus;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.runners.MockitoJUnitRunner;
import org.opengroup.osdu.register.utils.OpenApiFileUtils;
import org.opengroup.osdu.register.ddms.model.Ddms;
import org.opengroup.osdu.register.provider.interfaces.ddms.IDdmsRepository;
import org.opengroup.osdu.register.ddms.services.ConsumptionService;
import org.opengroup.osdu.core.common.model.http.AppException;
import org.opengroup.osdu.register.logging.AuditLog;
import org.springframework.http.ResponseEntity;

import java.net.URI;
import java.util.Collections;
import java.util.List;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.fail;
import static org.mockito.Matchers.any;
import static org.mockito.Matchers.anyString;
import static org.mockito.Mockito.verify;
import static org.mockito.Mockito.when;

@RunWith(MockitoJUnitRunner.class)
public class DdmsApiTest {

    @Mock
    private IDdmsRepository dmsRepository;
    @Mock
    private AuditLog auditLog;
    @Mock
    private ConsumptionService consumptionService;
    @InjectMocks
    private DdmsApi sut;

    @Test
    public void should_return307WithDmsLocation_when_givenValidParams() throws Exception {
        URI uri = new URI("https://ddms-domain-demo-dot-opendes.appspot.com/wellbore/123435ed");
        when(this.consumptionService.getSingleEntityRetrivalUri(anyString(), anyString(), anyString())).thenReturn(uri);

        ResponseEntity response = this.sut.redirectToDms(anyString(), anyString(), anyString());

        assertEquals(HttpStatus.SC_TEMPORARY_REDIRECT, response.getStatusCodeValue());
        assertEquals(uri, response.getHeaders().getLocation());
    }

    @Test
    public void should_return201_createDdms_when_givenValidSpec() throws Exception {
        Ddms model = OpenApiFileUtils.getDmsWithOpenSpec("validSubsurfaceDdms.json", "testType");

        ResponseEntity response = this.sut.postDMS(model);

        assertEquals(HttpStatus.SC_CREATED, response.getStatusCodeValue());
        verify(this.auditLog).createDmsSuccess(model.getId());
    }

    @Test
    public void should_return200_getDdms_when_givenValidId() {
        ResponseEntity response = this.sut.getDMS(any());

        assertEquals(HttpStatus.SC_OK, response.getStatusCodeValue());
    }

    @Test
    public void should_return204_deleteDdms_when_givenValidId() {
        String id = "1560392249745";
        when(this.dmsRepository.delete(id)).thenReturn(true);

        ResponseEntity response = this.sut.deleteDMS(id);

        assertEquals(HttpStatus.SC_NO_CONTENT, response.getStatusCodeValue());
        verify(this.auditLog).deleteDmsSuccess(id);
    }

    @Test
    public void should_return404_deleteDdms_when_givenInvalidId() {
        String id = "1560392249745";
        when(this.dmsRepository.delete(id)).thenReturn(false);

        try {
            this.sut.deleteDMS(id);
            fail("Should not succeed!");
        } catch (AppException e) {
            assertEquals(HttpStatus.SC_NOT_FOUND, e.getError().getCode());
            assertEquals(String.format("Could not delete DDMS with id %s because it could not be found", id), e.getError().getMessage());
            assertEquals("Not found", e.getError().getReason());
        } catch (Exception e) {
            fail("Should not throw this exception " + e.getMessage());
        }
    }

    @Test
    public void should_return200WithEntites_when_givenQueryThatExists() {
        List<Ddms> expected = Collections.singletonList(new Ddms());
        when(dmsRepository.query("query")).thenReturn(expected);

        ResponseEntity response = this.sut.queryDMS("query");

        assertEquals(HttpStatus.SC_OK, response.getStatusCodeValue());
        assertEquals(expected, response.getBody());
    }
}