/*
 * Copyright 2017-2020, Schlumberger
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.opengroup.osdu.register.subscriber.model;

import org.opengroup.osdu.register.utils.Helpers;
import org.junit.Test;

public class SubscriberTest {

    @Test
    public void should_throwNoValidationErrors_when_allPropertiesAreOkForGsaType() {
        Subscription sut = validSubscription(true);
        Helpers.TestValidation(sut);
    }

    @Test
    public void should_throwNoValidationErrors_when_allPropertiesAreOkForHmacType() {
        Subscription sut = validSubscription(false);
        Helpers.TestValidation(sut);
    }

    @Test
    public void should_throwValidationError_when_HmacSecreteValueHasInvalidChar() {
        Subscription sut = validSubscription(false);
        HmacSecret secret = new HmacSecret();
        secret.setValue("abc%123");
        sut.setSecret(secret);

        Helpers.TestValidation(sut, "HMAC value must match \"^[a-zA-Z0-9]{8,30}+$\" and length should be even.");
    }

    @Test
    public void should_throwValidationError_when_HmacSecreteValueLengthIsNotEven() {
        Subscription sut = validSubscription(false);

        HmacSecret secret = new HmacSecret();
        secret.setValue("abc123de1");
        sut.setSecret(secret);

        Helpers.TestValidation(sut, "HMAC value must match \"^[a-zA-Z0-9]{8,30}+$\" and length should be even.");
    }

    @Test
    public void should_throwValidationError_when_GsaSecreteValueMissingAudience() {
        Subscription sut = validSubscription(true);

        GsaSecret secret = new GsaSecret();
        GsaSecretValue secretValue = new GsaSecretValue("", "key");
        secret.setValue(secretValue);
        sut.setSecret(secret);

        Helpers.TestValidation(sut, "audience & key are required for GSA secret type");
    }

    @Test
    public void should_throwValidationError_when_GsaSecreteValueMissingAccountKey() {
        Subscription sut = validSubscription(true);

        GsaSecret secret = new GsaSecret();
        GsaSecretValue secretValue = new GsaSecretValue("audience", "");
        secret.setValue(secretValue);
        sut.setSecret(secret);

        Helpers.TestValidation(sut, "audience & key are required for GSA secret type");
    }

    @Test
    public void should_throwValidationError_when_nameIsInvalid() {
        Subscription sut = validSubscription(true);

        sut.setName("abc%123abc%123abc%123abc%123abc%123abc%123abc%123abc%123abc");

        Helpers.TestValidation(sut, "must match \"^[A-Za-z0-9- ]{2,50}\"");
    }

    private Subscription validSubscription(boolean gsaType) {
        Subscription dto = new Subscription();
        dto.setId("123");
        dto.setName("123");
        dto.setDescription("123");
        dto.setTopic("Test topic");
        dto.setPushEndpoint("http://localhost:8080/api/register/v1/test/challenge");

        if (gsaType) {
            GsaSecret secret = new GsaSecret();
            secret.setValue(new GsaSecretValue("audience", "key"));
            dto.setSecret(secret);
        } else {
            HmacSecret secret = new HmacSecret();
            secret.setValue("hmacEvenChar");
            dto.setSecret(secret);
        }
        return dto;
    }
}
